import React, { useState, useEffect, useRef } from 'react';
import { TextField, Button, Container, Avatar, Tooltip, Alert, CircularProgress } from '@mui/material';
import { PRIMARY_COLOR, SECONDARY_COLOR } from '../App';
import { useApiCall } from './hooks/useApiCall';
import { api } from '../services/api';

export default function ItemDetails({ item, token, onSave, onClose }) {
  const [name, setName] = useState(item.name);
  const [description, setDescription] = useState(item.description);
  const [imagePath, setImagePath] = useState(item.image_path || '');
  const [imageSrc, setImageSrc] = useState('/images/default.jpg');
  const [loading, setLoading] = useState(true);
  const [error, setError] = useState(null);
  const fileInputRef = useRef(null);
  const [imageOverlayVisible, setImageOverlayVisible] = useState(false);
  const [boxes, setBoxes] = useState([]);
  const [selectedBoxId, setSelectedBoxId] = useState(item.box_id);

  const { execute: fetchBoxes, loading: loadingBoxes, error: boxesError } = useApiCall();
  const { execute: updateItem, loading: savingItem, error: saveError } = useApiCall();
  const { execute: uploadImage, loading: uploadingImage, error: uploadError } = useApiCall();

  useEffect(() => {
    const getBoxes = async () => {
      try {
        const response = await fetchBoxes(() => api.boxes.getAll(token));
        setBoxes(response.data);
      } catch (err) {}
    };
    getBoxes();
  }, [token]);

  useEffect(() => {
    const loadImage = async () => {
      setLoading(true);
      setError(null);
      try {
        const response = await api.items.getImage(token, item.ID);
        const reader = new FileReader();
        reader.onload = () => {
          setImageSrc(reader.result);
          setLoading(false);
        };
        reader.readAsDataURL(response.data);
      } catch (err) {
        setImageSrc('/default.jpg');
        setError(err);
        setLoading(false);
      }
    };
    loadImage();
  }, [item.ID, token]);

  const handleImageUpload = async () => {
    if (!fileInputRef.current?.files?.[0]) return null;

    const formData = new FormData();
    formData.append('image', fileInputRef.current.files[0]);

    try {
      const response = await uploadImage(() => 
        api.items.uploadImage(token, item.ID, formData)
      );
      return response.data.imagePath;
    } catch (err) {
      return null;
    }
  };

  const handleSave = async () => {
    if (fileInputRef.current?.files?.[0]) {
      await handleImageUpload();
    }

    try {
      await updateItem(() => api.items.update(token, item.ID, {
        name,
        description,
        box_id: +selectedBoxId,
      }));
      onSave();
    } catch (err) {}
  };

  const handleBoxChange = (event) => {
    setSelectedBoxId(event.target.value);
  };

  return (
    <Container>
      <h3>Edit Item: {item.name}</h3>
      
      {(error || boxesError || saveError || uploadError) && (
        <Alert severity="error" sx={{ mb: 2 }}>
          {error?.message || boxesError?.message || saveError?.message || uploadError?.message}
        </Alert>
      )}

      {loading ? (
        <CircularProgress />
      ) : (
        <Tooltip title="Click to enlarge">
          <Avatar
            src={imageSrc}
            alt={name}
            sx={{ width: 200, height: 200, marginBottom: '16px' }}
            onClick={() => setImageOverlayVisible(true)}
          />
        </Tooltip>
      )}

      {imageOverlayVisible && (
        <div className="image-overlay">
          <img src={imageSrc} alt={name} />
          <button className="close-button" onClick={() => setImageOverlayVisible(false)}>
            Close
          </button>
        </div>
      )}

      <TextField
        label="Item Name"
        variant="outlined"
        fullWidth
        margin="normal"
        value={name}
        onChange={(e) => setName(e.target.value)}
        disabled={savingItem}
      />
      <TextField
        label="Item Description"
        variant="outlined"
        fullWidth
        margin="normal"
        value={description}
        onChange={(e) => setDescription(e.target.value)}
        disabled={savingItem}
      />

      {loadingBoxes ? (
        <CircularProgress size={24} />
      ) : (
        <select 
          value={selectedBoxId} 
          onChange={handleBoxChange}
          disabled={savingItem}
        >
          <option value="">No box</option>
          {boxes.map((box) => (
            <option key={box.ID} value={box.ID}>
              {box.name}
            </option>
          ))}
        </select>
      )}

      <br />
      <br />
      
      <Button 
        sx={{ backgroundColor: PRIMARY_COLOR, borderBottom: '1px solid', borderColor: '#444', color: SECONDARY_COLOR }}
        variant="contained" 
        component="label"
        disabled={uploadingImage || savingItem}
      >
        {uploadingImage ? <CircularProgress size={24} /> : 'Upload Image'}
        <input
          type="file"
          accept="image/*"
          ref={fileInputRef}
          style={{ display: 'none' }}
          onChange={(e) => setImagePath(e.target.files[0]?.name || '')}
        />
      </Button>

      <Button 
        sx={{ backgroundColor: PRIMARY_COLOR, borderBottom: '1px solid', borderColor: '#444', color: SECONDARY_COLOR }}
        variant="contained"
        onClick={handleSave}
        disabled={savingItem}
      >
        {savingItem ? <CircularProgress size={24} /> : 'Save Changes'}
      </Button>

      <Button 
        sx={{ backgroundColor: PRIMARY_COLOR, borderBottom: '1px solid', borderColor: '#444', color: SECONDARY_COLOR }}
        variant="contained"
        onClick={onClose}
        disabled={savingItem}
      >
        Close
      </Button>
    </Container>
  );
}