import { useState } from 'react';

/**
 * useApiCall hook.
 *
 * This hook helps to wrap API calls in a way that makes it easier to manage
 * loading state and errors.
 *
 * @returns {Object} An object with `execute`, `loading`, and `error`
 * properties. `execute` is a function that wraps the API call and sets the
 * `loading` and `error` states accordingly. `loading` is a boolean that is
 * `true` while the API call is in progress and `false` otherwise. `error` is
 * `null` if the API call was successful and an error object if the API call
 * failed.
 *
 * @example
 * const { execute, loading, error } = useApiCall();
 *
 * const fetchData = async () => {
 *   const response = await execute(api.items.getAll());
 *   // Do something with the response
 * };
 */
export const useApiCall = () => {
    const [loading, setLoading] = useState(false);
    const [error, setError] = useState(null);
  
    const execute = async (apiCall) => {
      setLoading(true);
      setError(null);
      try {
        const result = await apiCall();
        return result;
      } catch (err) {
        setError(err);
        throw err;
      } finally {
        setLoading(false);
      }
    };
  
    return { execute, loading, error };
  };